﻿using DoubleDeckerBus.Entities;
using DoubleDeckerBus.MovementStrategy;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DoubleDeckerBus.DrawningObjects
{
    public class DrawningBus
    {
        public EntityBus? EntityBus { get; protected set; }
        public IMoveableObject GetMoveableObject => new DrawningObjectBus(this);
        public int? _pictureWidth;
        public int? _pictureHeight;
        protected int _startPosX;
        protected int _startPosY;
        protected readonly int _busWidth = 110;
        protected readonly int _busHeight = 70;
        public int GetPosX => _startPosX;
        public int GetPosY => _startPosY;
        public int GetWidth => _busWidth;
        public int GetHeight => _busHeight;
        public DrawningBus(int speed, float weight, Color bodyColor, int width, int height)
        {
            if (width < _busWidth || height < _busHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            EntityBus = new EntityBus(speed, weight, bodyColor);
        }
        protected DrawningBus(int speed, float weight, Color bodyColor, int width, int height, int busWidth, int busHeight)
        {
            if (width <= _busWidth || height <= _busHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            _busWidth = busWidth;
            _busHeight = busHeight;
            EntityBus = new EntityBus(speed, weight, bodyColor);
        }
        public void SetPosition(int x, int y)
        {
            if (_pictureWidth <= _busWidth || _pictureHeight <= _busHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            _startPosX = x;
            _startPosY = y;
        }
        public bool CanMove(DirectionBus direction)
        {
            if (EntityBus == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                DirectionBus.Left => _startPosX - EntityBus.Step > 0,
                //вверх
                DirectionBus.Up => _startPosY - EntityBus.Step > 0,
                //вправо
                DirectionBus.Right => _startPosX + _busWidth + EntityBus.Step < _pictureWidth,
                //вниз
                DirectionBus.Down => _startPosY + _busHeight + EntityBus.Step < _pictureHeight,
                _ => false,
            };
        }
        public void MoveTransport(DirectionBus direction)
        {
            if (!CanMove(direction) || EntityBus == null)
            {
                return;
            }
            switch (direction)
            { 
                //влево
                case DirectionBus.Left:
                    _startPosX -= (int)EntityBus.Step;
                    break;
                //вверх
                case DirectionBus.Up:
                    _startPosY -= (int)EntityBus.Step;
                    break;
                // вправо
                case DirectionBus.Right:
                    _startPosX += (int)EntityBus.Step;
                    break;
                //вниз
                case DirectionBus.Down:
                    _startPosY += (int)EntityBus.Step;
                    break;
            }
        }
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityBus == null)
            {
                return;
            }
            if (EntityBus == null)
            {
                return;
            }

            Pen pen = new(Color.Black);
            
            g.DrawRectangle(pen, _startPosX - 1, _startPosY + 11, 100, 30);
            Brush brBodyColor = new SolidBrush(EntityBus.BodyColor);
            g.FillRectangle(brBodyColor, _startPosX, _startPosY + 10, 100, 30);

        
            g.DrawRectangle(pen, _startPosX + 30, _startPosY + 20, 10, 20);
            Brush brBlack = new SolidBrush(Color.Black);
            g.FillRectangle(brBlack, _startPosX + 30, _startPosY + 20, 10, 20);

           
            g.DrawEllipse(pen, _startPosX + 7, _startPosY + 35, 10, 10);
            g.DrawEllipse(pen, _startPosX + 77, _startPosY + 35, 10, 10);
            g.FillEllipse(brBlack, _startPosX + 7, _startPosY + 35, 10, 10);
            g.FillEllipse(brBlack, _startPosX + 77, _startPosY + 35, 10, 10);

            Brush brBlue = new SolidBrush(Color.Blue);
            g.FillEllipse(brBlue, _startPosX + 10, _startPosY + 15, 10, 15);
            g.FillEllipse(brBlue, _startPosX + 50, _startPosY + 15, 10, 15);
            g.FillEllipse(brBlue, _startPosX + 70, _startPosY + 15, 10, 15);
            g.FillEllipse(brBlue, _startPosX + 90, _startPosY + 15, 10, 15);   
        }
    }
}
