﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DoubleDeckerBus.DrawningObjects;
using DoubleDeckerBus.MovementStrategy;

namespace DoubleDeckerBus.Generics
{
    internal class BusesGenericStorage
    {
        readonly Dictionary<string, BusesGenericCollection<DrawningBus, DrawningObjectBus>> _busStorages;
        public List<string> Keys => _busStorages.Keys.ToList();
        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        private static readonly char _separatorForKeyValue = '|';
        private readonly char _separatorRecords = ';';
        private static readonly char _separatorForObject = ':';
        public BusesGenericStorage(int pictureWidth, int pictureHeight)
        {
            _busStorages = new Dictionary<string, BusesGenericCollection<DrawningBus, DrawningObjectBus>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        public void AddSet(string name)
        {
            _busStorages.Add(name, new BusesGenericCollection<DrawningBus, DrawningObjectBus>(_pictureWidth, _pictureHeight));
        }
        public void DelSet(string name)
        {
            if (!_busStorages.ContainsKey(name))
            {
                return;
            }

            _busStorages.Remove(name);
        }
        public BusesGenericCollection<DrawningBus, DrawningObjectBus>?this[string ind]
        {
            get
            {
                if (_busStorages.ContainsKey(ind))
                {
                    return _busStorages[ind];
                }

                return null;
            }
        }
        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<string,
            BusesGenericCollection<DrawningBus, DrawningObjectBus>> record in _busStorages)
            {
                StringBuilder records = new();
                foreach (DrawningBus? elem in record.Value.GetTheBuses)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                throw new Exception("Невалидная операция, нет данных для сохранения");
            }
            using FileStream fs = new(filename, FileMode.Create);
            byte[] info = new
            UTF8Encoding(true).GetBytes($"BusStorage{Environment.NewLine}{data}");
            fs.Write(info, 0, info.Length);
            return;
        }
        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new Exception("Файл не найден");
            }
            string bufferTextFromFile = "";
            using (FileStream fs = new(filename, FileMode.Open))
            {
                byte[] b = new byte[fs.Length];
                UTF8Encoding temp = new(true);
                while (fs.Read(b, 0, b.Length) > 0)
                {
                    bufferTextFromFile += temp.GetString(b);
                }
            }
            var strs = bufferTextFromFile.Split(new char[] { '\n', '\r' },
            StringSplitOptions.RemoveEmptyEntries);
            if (strs == null || strs.Length == 0)
            {
                throw new Exception("Нет данных для загрузки");
            }
            if (!strs[0].StartsWith("BusStorage"))
            {
                //если нет такой записи, то это не те данные
                throw new Exception("Неверный формат данных");
            }
            _busStorages.Clear();
            foreach (string data in strs)
            {
                string[] record = data.Split(_separatorForKeyValue,
                StringSplitOptions.RemoveEmptyEntries);
                if (record.Length != 2)
                {
                    continue;
                }
                BusesGenericCollection<DrawningBus, DrawningObjectBus>
                collection = new(_pictureWidth, _pictureHeight);
                string[] set = record[1].Split(_separatorRecords,
                StringSplitOptions.RemoveEmptyEntries);
                foreach (string elem in set)
                {
                    DrawningBus? car =
                    elem?.CreateDrawningBus(_separatorForObject, _pictureWidth, _pictureHeight);
                    if (car != null)
                    {
                        if (!(collection + car))
                        {
                            throw new Exception("Ошибка добавления в коллекцию");
                        }
                    }
                }
                _busStorages.Add(record[0], collection);
            }
        }
    }
}

  

